import 'package:http/http.dart' as http;
import 'dart:convert';
import 'package:web_socket_channel/web_socket_channel.dart';
import 'dart:async';
import 'dart:io';
class ApiClient {
  final String dataBaseUrl = "https://data.mentor.iblai.tech";
  final String authToken;

  ApiClient({required this.authToken});

  Map<String, String> _headers() {
    return {
      'Content-Type': 'application/json',
      'Authorization': 'Token $authToken',
    };
  }

  Map<String, String> _multipartHeaders() {
    return {
      'Authorization': 'Token $authToken',
    };
  }
  Future<dynamic> apiAiAccountOrgsOrgCredentialGet(String? org) async {
    final response = await http.get(
      Uri.parse('$dataBaseUrl/api/ai/account/orgs/$org/credential/'),
      headers: _headers(),
    );
    return json.decode(response.body);
  }

  Future<dynamic> apiAiAccountOrgsOrgCredentialPost(String? org, Map body) async {
    final response = await http.post(
      Uri.parse('$dataBaseUrl/api/ai/account/orgs/$org/credential/'),
      headers: _headers(),
      body: json.encode(body),
    );
    return json.decode(response.body);
  }

  Future<dynamic> apiAiAccountOrgsOrgCredentialPatch(String? org, Map body) async {
    final response = await http.patch(
      Uri.parse('$dataBaseUrl/api/ai/account/orgs/$org/credential/'),
      headers: _headers(),
      body: json.encode(body),
    );
    return json.decode(response.body);
  }

  Future<dynamic> apiAiAccountOrgsOrgPlatformMetadataGet(String? org) async {
    final response = await http.get(
      Uri.parse('$dataBaseUrl/api/ai/account/orgs/$org/platform-metadata/'),
      headers: _headers(),
    );
    return json.decode(response.body);
  }

  Future<dynamic> apiAiAccountOrgsOrgPlatformMetadataPost(String? org, Map body) async {
    final response = await http.post(
      Uri.parse('$dataBaseUrl/api/ai/account/orgs/$org/platform-metadata/'),
      headers: _headers(),
      body: json.encode(body),
    );
    return json.decode(response.body);
  }

  Future<dynamic> apiAiAccountOrgsOrgPlatformMetadataPut(String? org, Map body) async {
    final response = await http.put(
      Uri.parse('$dataBaseUrl/api/ai/account/orgs/$org/platform-metadata/'),
      headers: _headers(),
      body: json.encode(body),
    );

    return json.decode(response.body);
  }

  Future<dynamic> apiAiAccountOrgsOrgTokensGet(String? org) async {
    final response = await http.get(
      Uri.parse('$dataBaseUrl/api/ai/account/orgs/$org/tokens/'),
      headers: _headers(),
    );
    return json.decode(response.body);
  }

  Future<dynamic> apiAiIndexOrgsOrgUsersUserIdDocumentsDocumentIdRetrieverDocumentsGet(String? documentId, String? org, String? userId) async {
    final response = await http.get(
      Uri.parse('$dataBaseUrl/api/ai/index/orgs/$org/users/$userId/documents/$documentId/retriever_documents/'),
    );
    return json.decode(response.body);
  }

  Future<dynamic> apiAiIndexOrgsOrgUsersUserIdDocumentsDocumentIdRetrieverDocumentsPut(String? documentId, String? org, String? userId, Map body) async {
    final response = await http.put(
      Uri.parse('$dataBaseUrl/api/ai/index/orgs/$org/users/$userId/documents/$documentId/retriever_documents/'),
      body: json.encode(body),
      headers: _headers(),
    );
    return json.decode(response.body);
  }

  Future<dynamic> apiAiIndexOrgsOrgUsersUserIdMultilingualRetrieverTrainPost(String? org, String? userId, Map body) async {
    final response = await http.post(
      Uri.parse('$dataBaseUrl/api/ai/index/orgs/$org/users/$userId/multilingual_retriever_train/'),
      body: json.encode(body),
      headers: _headers(),
    );
    return json.decode(response.body);
  }

  Future<dynamic> apiAiIndexOrgsOrgUsersUserIdPathwaysPathwayRetrieverDocumentsGet(String? org, String? pathway, String? userId) async {
    final response = await http.get(
      Uri.parse('$dataBaseUrl/api/ai/index/orgs/$org/users/$userId/pathways/$pathway/retriever_documents/'),
      headers: _headers(),
    );
    return json.decode(response.body);
  }

  Future<dynamic> apiAiIndexOrgsOrgUsersUserIdRetrieverSearchPost(String? org, String? userId, Map body) async {
    final response = await http.post(
      Uri.parse('$dataBaseUrl/api/ai/index/orgs/$org/users/$userId/retriever_search/'),
      body: json.encode(body),
      headers: _headers(),
    );
    return json.decode(response.body);
  }

  Future<void> deleteChatSession(String org, String userId, String sessionId) async {
    final response = await http.delete(
      Uri.parse('$dataBaseUrl/api/ai/mentor/orgs/$org/users/$userId/sessions/$sessionId/'),
      headers: _headers(),
    );

    if (response.statusCode == 204) {
      // Successfully deleted with no content in the response body
      return;
    } else {
      // Handle other status codes or errors
      throw Exception('Failed to delete chat session. Status code: ${response.statusCode}');
    }
  }

Future<dynamic> updateChatSessionDetails(String? org, String? userId, String? sessionId, String? title) async {
  // Check if any of the URL parameters are null
  if (org == null || userId == null || sessionId == null) {
    throw Exception('Missing required parameters: org, userId, or sessionId');
  }

  // Construct the URL with the provided parameters
  String url = '$dataBaseUrl/api/ai/mentor/orgs/$org/users/$userId/sessions/$sessionId/';

  // Prepare the headers
  Map<String, String> headers = _headers();

  // Prepare the body data
  Map<String, dynamic> bodyData = {
    "title": title,
    // Include any other fields required by your API here
  };

  try {
    // Make the HTTP PUT request
    final response = await http.put(
      Uri.parse(url),
      headers: headers,
      body: json.encode(bodyData),
    );

    // Check the response status code
    if (response.statusCode == 200) {
      return json.decode(response.body);
    } else {
      throw Exception('Failed to update chat session details. Status code: ${response.statusCode}');
    }
  } catch (error) {
    // Handle any errors that occur during the request
    throw Exception('Error occurred during HTTP request: $error');
  }
}

  Future<dynamic> apiAiIndexOrgsOrgUsersUserIdRetrieverTrainPost(String? org, String? userId, Map body) async {
    final response = await http.post(
      Uri.parse('$dataBaseUrl/api/ai/index/orgs/$org/users/$userId/retriever_train/'),
      body: json.encode(body),
      headers: _headers(),
    );
    return json.decode(response.body);
  }

  Future<dynamic> apiAiIndexOrgsOrgUsersUserIdSourcesPost(String? org, String? userId, Map body) async {
    final response = await http.post(
      Uri.parse('$dataBaseUrl/api/ai/index/orgs/$org/users/$userId/sources/'),
      body: json.encode(body),
      headers: _headers(),
    );
    return json.decode(response.body);
  }

  Future<dynamic> apiAiIndexOrgsOrgUsersUserIdTasksTaskIdGet(String? org, String? taskId, String? userId) async {
    final response = await http.get(
      Uri.parse('$dataBaseUrl/api/ai/index/orgs/$org/users/$userId/tasks/$taskId/'),
      headers: _headers(),
    );
    return json.decode(response.body);
  }

  Future<dynamic> apiAiMentorOrgsOrgUsersUserIdGet({
    required String? org,
    int? page,
    int? pageSize,
    required String? userId,
    bool? featured, // Optional featured parameter
  }) async {
    var queryParams = {
      if (page != null) 'page': page.toString(),
      if (pageSize != null) 'pageSize': pageSize.toString(),
      if (featured != null) 'featured': featured.toString(), // Add featured to query params if not null
    };

    var queryString = Uri(queryParameters: queryParams).query;

    final response = await http.get(
      Uri.parse('$dataBaseUrl/api/ai/mentor/orgs/$org/users/$userId/?$queryString'),
      headers: _headers(),
    );
    return json.decode(response.body);
  }

  Future<dynamic> apiAiMentorOrgsOrgMentorWithSettingsPost(String? org, Map body) async {
    final response = await http.post(
      Uri.parse('$dataBaseUrl/api/ai/mentor/orgs/$org/mentor-with-settings/'),
      headers: _headers(),
      body: json.encode(body),
    );
    return json.decode(response.body);
  }

  Future<List<dynamic>> websocket({
  required String baseDataUrl,
  required String mentor,
  required String sessionId,
  required String token,
  required String username,
  String? prompt, // Optional prompt
}) async {
  var channel = WebSocketChannel.connect(
    Uri.parse('wss://$baseDataUrl/ws/langflow/'),
  );

  final controller = StreamController<List<dynamic>>();
  List<dynamic> responses = [];

  // Initial data without the prompt
  final dataWithoutPrompt = {
    "flow": {"name": mentor, "tenant": 'main', "username": username},
    "session_id": sessionId,
    "token": token
  };

  // Send initial data without prompt
  channel.sink.add(json.encode(dataWithoutPrompt));

  // Listen for messages
  channel.stream.listen((message) {
    final data = jsonDecode(message);

    // Handle the server response
    if (data['status'] == 'connected') {
      responses.add(data);

      if (prompt != null) {
        // Send the second message with the prompt
        final dataWithPrompt = {...dataWithoutPrompt, "prompt": prompt};
        channel.sink.add(json.encode(dataWithPrompt));
        prompt = null; // Prevent sending the prompt again
      } else {
        // If no prompt was initially provided, add responses and close the connection
        controller.add(responses);
        channel.sink.close();
      }
    }
  }, onDone: () {
    controller.close(); // Close the controller when the stream is done
  }, onError: (error) {
    controller.addError('Error connecting to server: $error');
    channel.sink.close(); // Close the WebSocket connection on error
  });

  return controller.stream.first; // Return the first (and only) item from the stream
}


  Future<dynamic> apiAiMentorOrgsOrgMentorsMentorSuggestedPromptsGet({
    required String? mentor,
    required String? org,
  }) async {
    final response = await http.get(
      Uri.parse('$dataBaseUrl/api/ai/mentor/orgs/$org/mentors/$mentor/suggested-prompts/'),
      headers: _headers(),
    );
    return json.decode(response.body);
  }
  Future<dynamic> apiAiMentorOrgsOrgMentorsMentorPromptsGet({
    required String? org,
    required String? user_id,
  }) async {
    final response = await http.get(
      Uri.parse('$dataBaseUrl/api/ai/mentor/orgs/$org/users/$user_id/prompt/'),
      headers: _headers(),
    );
    return json.decode(response.body);
  }

  Future<dynamic> apiAiMentorOrgsOrgUsersUserIdRecentMessagesGet({
    required String? org,
    required String? userId,
  }) async {
    final response = await http.get(
      Uri.parse('$dataBaseUrl/api/ai/mentor/orgs/$org/users/$userId/recent-messages/'),
      headers: _headers(),
    );
    return json.decode(response.body);
  }

  // Function for Session ID
  Future<dynamic> apiAiMentorOrgsOrgUsersUserIdSessionidGet({
    required String? endDate,
    required String? org,
    required String? startDate,
    required String? userId,
  }) async {
    var queryParams = {
      'endDate': endDate,
      'startDate': startDate,
    };

    var queryString = Uri(queryParameters: queryParams).query;

    final response = await http.get(
      Uri.parse('$dataBaseUrl/api/ai/mentor/orgs/$org/users/$userId/sessionid/?$queryString'),
      headers: _headers(),
    );
    return json.decode(response.body);
  }
// Getting pin of messages

  Future<dynamic> apiAiMentorOrgsOrgUsersUserIdPinMessageGet({
    required String org,
    required String userId,
  }) async {
    final response = await http.get(
      Uri.parse('$dataBaseUrl/api/ai/mentor/orgs/$org/users/$userId/pin-message/'),
      headers: _headers(),
    );
    return json.decode(response.body);
  }

  Future<dynamic> apiAiMentorShareSession({
    required String org,
    required String userId,
    required String session_id,
  }) async {
    final response = await http.get(
      Uri.parse('$dataBaseUrl/api/ai/mentor/orgs/$org/users/$userId/sessions/$session_id/?shared=true'),
      headers: _headers(),
    );
    return json.decode(response.body);
  }

  Future<dynamic> trainSessionDocument({
    required String org,
    required String userId,
    required String sessionId,
    required File file,
  }) async {
    var uri = Uri.parse('$dataBaseUrl/api/ai/index/orgs/$org/users/$userId/train-session-document/sessions/$sessionId/');
    var request = http.MultipartRequest('POST', uri)
      ..headers.addAll(_multipartHeaders()) // Use the multipart headers here
      ..files.add(await http.MultipartFile.fromPath(
        'file', 
        file.path,
      ));

    var streamedResponse = await request.send();
    var response = await http.Response.fromStream(streamedResponse);

    if (response.statusCode == 200) {
      return json.decode(response.body);
    } else {
      throw Exception('Failed to upload file. Status code: ${response.statusCode}');
    }
  }

  // Create a session

  Future<dynamic> apiAiMentorOrgsOrgUsersUserIdSessionsPost({
    required String org,
    required String userId,
    required Map<String, dynamic> body,
  }) async {
    final response = await http.post(
      Uri.parse('$dataBaseUrl/api/ai/mentor/orgs/$org/users/$userId/sessions/'),
      headers: _headers(),
      body: json.encode(body),
    );

    if (response.statusCode >= 200 && response.statusCode < 300) {
      return json.decode(response.body);
    } else {
      throw Exception('Failed to post session. Status code: ${response.statusCode}');
    }
  }
  // Create prompts

  Future<dynamic> apiAiMentorOrgsOrgMentorsMentorSuggestedPromptsPost({
    required String? mentor,
    required String? org,
    required Map<String, dynamic>? body, // Assuming JSON body
  }) async {
    final response = await http.post(
      Uri.parse('$dataBaseUrl/api/ai/mentor/orgs/$org/mentors/$mentor/suggested-prompts/'),
      headers: _headers(),
      body: json.encode(body), // Encode body to JSON string
    );

    if (response.statusCode == 200) {
      // Handle successful response
      return json.decode(response.body);
    } else {
      // Handle error
      throw Exception('Failed to post suggested prompts');
    }
  }
}
